--- old/usr/src/lib/libzfs/common/libzfs_dataset.c	2013-04-25 08:45:55.944225765 +0200
+++ new/usr/src/lib/libzfs/common/libzfs_dataset.c	2013-04-25 08:45:55.482611122 +0200
@@ -24,6 +24,7 @@
  * Copyright (c) 2012 by Delphix. All rights reserved.
  * Copyright (c) 2012 DEY Storage Systems, Inc.  All rights reserved.
  * Copyright 2012 Nexenta Systems, Inc. All rights reserved.
+ * Copyright (c) 2013 Martin Matuska. All rights reserved.
  */
 
 #include <ctype.h>
@@ -4116,6 +4117,20 @@
 	ha.tag = tag;
 	ha.recursive = recursive;
 	(void) zfs_hold_one(zfs_handle_dup(zhp), &ha);
+
+	if (nvlist_next_nvpair(ha.nvl, NULL) == NULL) {
+		fnvlist_free(ha.nvl);
+		ret = ENOENT;
+		if (!enoent_ok) {
+			(void) snprintf(errbuf, sizeof (errbuf),
+			    dgettext(TEXT_DOMAIN,
+			    "cannot hold snapshot '%s@%s'"),
+			    zhp->zfs_name, snapname);
+			(void) zfs_standard_error(hdl, ret, errbuf);
+		}
+		return (ret);
+	}
+
 	ret = lzc_hold(ha.nvl, cleanup_fd, &errors);
 	fnvlist_free(ha.nvl);
 
@@ -4217,12 +4232,25 @@
 	nvlist_t *errors;
 	nvpair_t *elem;
 	libzfs_handle_t *hdl = zhp->zfs_hdl;
+	char errbuf[1024];
 
 	ha.nvl = fnvlist_alloc();
 	ha.snapname = snapname;
 	ha.tag = tag;
 	ha.recursive = recursive;
 	(void) zfs_release_one(zfs_handle_dup(zhp), &ha);
+
+	if (nvlist_next_nvpair(ha.nvl, NULL) == NULL) {
+		fnvlist_free(ha.nvl);
+		ret = ENOENT;
+		(void) snprintf(errbuf, sizeof (errbuf),
+		    dgettext(TEXT_DOMAIN,
+		    "cannot release hold from snapshot '%s@%s'"),
+		    zhp->zfs_name, snapname);
+		(void) zfs_standard_error(hdl, ret, errbuf);
+		return (ret);
+	}
+
 	ret = lzc_release(ha.nvl, &errors);
 	fnvlist_free(ha.nvl);
 
@@ -4231,8 +4259,6 @@
 
 	if (nvlist_next_nvpair(errors, NULL) == NULL) {
 		/* no hold-specific errors */
-		char errbuf[1024];
-
 		(void) snprintf(errbuf, sizeof (errbuf), dgettext(TEXT_DOMAIN,
 		    "cannot release"));
 		switch (errno) {
@@ -4249,8 +4275,6 @@
 	for (elem = nvlist_next_nvpair(errors, NULL);
 	    elem != NULL;
 	    elem = nvlist_next_nvpair(errors, elem)) {
-		char errbuf[1024];
-
 		(void) snprintf(errbuf, sizeof (errbuf),
 		    dgettext(TEXT_DOMAIN,
 		    "cannot release hold from snapshot '%s'"),
